@@ -535,6 +535,14 @@ def net_is_vlan(context, name, inventory_hostname=None):
     return net_vlan(context, name) is not None
 
 
+@jinja2.contextfilter
+def net_is_vlan_interface(context, name, inventory_hostname=None):
+    device = get_and_validate_interface(context, name, inventory_hostname)
+    # Use a heuristic to match conventional VLAN names, ending with a
+    # period and a numerical extension to an interface name
+    return re.match(r"^[a-zA-Z0-9_\-]+\.[1-9][\d]{0,3}$", device)
+
+
 @jinja2.contextfilter
 def net_select_ethers(context, names, inventory_hostname=None):
     return [name for name in names
@@ -559,6 +567,12 @@ def net_select_vlans(context, names, inventory_hostname=None):
             if net_is_vlan(context, name, inventory_hostname)]
 
 
+@jinja2.contextfilter
+def net_select_vlan_interfaces(context, names, inventory_hostname=None):
+    return [name for name in names
+            if net_is_vlan_interface(context, name, inventory_hostname)]
+
+
 @jinja2.contextfilter
 def net_reject_vlans(context, names, inventory_hostname=None):
     return [name for name in names
